@@ -3,6 +3,9 @@ import sys
 import re
 import locale
 import shutil
+from itertools import islice
+from collections import namedtuple
+from numbers import Number
 from datetime import datetime, timezone
 from textwrap import TextWrapper
 from functools import lru_cache
@@ -151,8 +154,9 @@ def compatibility(mod_name: str, compatible: List[str], quiet: bool = False) ->
         mod = mod_name.split('.')[-1]
         compat_list = ', '.join(compatible)
         warning_message([
-            f'{mod} parser is not compatible with your OS ({sys.platform}).',
-            f'Compatible platforms: {compat_list}'
+            f'`{mod}` command output from this OS ({sys.platform}) is not supported.',
+            f'`{mod}` command output from the following platforms is supported: {compat_list}',
+            'Disregard this warning if you are processing output that came from a supported platform. (Use the -q option to suppress this warning)'
         ])
 
 
@@ -179,6 +183,68 @@ def has_data(data: Union[str, bytes]) -> bool:
     return bool(data)
 
 
+def remove_quotes(data: str) -> str:
+    """
+    Remove single or double quotes surrounding a string. If no quotes are
+    found then the string is returned unmodified.
+
+    Parameters:
+
+        data:       (string) Input value
+
+    Returns:
+
+        string
+    """
+    if data.startswith('"') and data.endswith('"'):
+        data = data[1:-1]
+
+    elif data.startswith("'") and data.endswith("'"):
+        data = data[1:-1]
+
+    return data
+
+
+def normalize_key(data: str) -> str:
+    """
+    Normalize a key name by shifting to lower-case and converting special
+    characters to underscores.
+
+    Special characters are defined as `space` and the following:
+
+        !"#$%&'()*+,-./:;<=>?@[\]^`{|}~
+
+    This is a lossy algorithm. Repeating and trailing underscores are
+    removed.
+
+    Parameters:
+
+        data:       (string) Input value
+
+    Returns:
+
+        string
+    """
+    special = '''!"#$%&'()*+,-./:;<=>?@[\]^`{|}~ '''
+    initial_underscore = False
+    data = data.strip().lower()
+
+    for special_char in special:
+        data = data.replace(special_char, '_')
+
+    if data.startswith('_'):
+        initial_underscore = True
+
+    # swap back to space so split() will compress multiple consecutive down to one
+    data = data.strip('_').replace('_', ' ')
+    data = '_'.join(data.split())
+
+    if initial_underscore:
+        data = '_' + data
+
+    return data
+
+
 def convert_to_int(value: object) -> Optional[int]:
     """
     Converts string and float input to int. Strips all non-numeric
@@ -273,12 +339,199 @@ def convert_to_bool(value: object) -> bool:
     return False
 
 
+# convert_size_to_int from https://github.com/xolox/python-humanfriendly
+
+# Copyright (c) 2021 Peter Odding
+
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
+
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+def convert_size_to_int(size: str, binary: bool = False) -> Optional[int]:
+    """
+    Parse a human readable data size and return the number of bytes.
+
+    Parameters:
+
+        size:           (string) The human readable file size to parse.
+        binary:         (boolean) `True` to use binary multiples of bytes
+                        (base-2) for ambiguous unit symbols and names,
+                        `False` to use decimal multiples of bytes (base-10).
+    Returns:
+
+        integer/None    Integer if successful conversion, otherwise None
+
+    This function knows how to parse sizes in bytes, kilobytes, megabytes,
+    gigabytes, terabytes and petabytes. Some examples:
+
+        >>> convert_size_to_int('42')
+        42
+        >>> convert_size_to_int('13b')
+        13
+        >>> convert_size_to_int('5 bytes')
+        5
+        >>> convert_size_to_int('1 KB')
+        1000
+        >>> convert_size_to_int('1 kilobyte')
+        1000
+        >>> convert_size_to_int('1 KiB')
+        1024
+        >>> convert_size_to_int('1 KB', binary=True)
+        1024
+        >>> convert_size_to_int('1.5 GB')
+        1500000000
+        >>> convert_size_to_int('1.5 GB', binary=True)
+        1610612736
+    """
+    def tokenize(text: str) -> List[str]:
+        tokenized_input: List = []
+        for token in re.split(r'(\d+(?:\.\d+)?)', text):
+            token = token.strip()
+            if re.match(r'\d+\.\d+', token):
+                tokenized_input.append(float(token))
+            elif token.isdigit():
+                tokenized_input.append(int(token))
+            elif token:
+                tokenized_input.append(token)
+        return tokenized_input
+
+    SizeUnit = namedtuple('SizeUnit', 'divider, symbol, name')
+    CombinedUnit = namedtuple('CombinedUnit', 'decimal, binary')
+    disk_size_units = (
+        CombinedUnit(SizeUnit(1000**1, 'KB', 'kilobyte'), SizeUnit(1024**1, 'KiB', 'kibibyte')),
+        CombinedUnit(SizeUnit(1000**2, 'MB', 'megabyte'), SizeUnit(1024**2, 'MiB', 'mebibyte')),
+        CombinedUnit(SizeUnit(1000**3, 'GB', 'gigabyte'), SizeUnit(1024**3, 'GiB', 'gibibyte')),
+        CombinedUnit(SizeUnit(1000**4, 'TB', 'terabyte'), SizeUnit(1024**4, 'TiB', 'tebibyte')),
+        CombinedUnit(SizeUnit(1000**5, 'PB', 'petabyte'), SizeUnit(1024**5, 'PiB', 'pebibyte')),
+        CombinedUnit(SizeUnit(1000**6, 'EB', 'exabyte'), SizeUnit(1024**6, 'EiB', 'exbibyte')),
+        CombinedUnit(SizeUnit(1000**7, 'ZB', 'zettabyte'), SizeUnit(1024**7, 'ZiB', 'zebibyte')),
+        CombinedUnit(SizeUnit(1000**8, 'YB', 'yottabyte'), SizeUnit(1024**8, 'YiB', 'yobibyte')),
+    )
+    tokens = tokenize(size)
+    if tokens and isinstance(tokens[0], Number):
+        # Get the normalized unit (if any) from the tokenized input.
+        normalized_unit = tokens[1].lower() if len(tokens) == 2 and isinstance(tokens[1], str) else ''
+        # If the input contains only a number, it's assumed to be the number of
+        # bytes. The second token can also explicitly reference the unit bytes.
+        if len(tokens) == 1 or normalized_unit.startswith('b'):
+            return int(tokens[0])
+        # Otherwise we expect two tokens: A number and a unit.
+        if normalized_unit:
+            # Convert plural units to singular units, for details:
+            # https://github.com/xolox/python-humanfriendly/issues/26
+            normalized_unit = normalized_unit.rstrip('s')
+            for unit in disk_size_units:
+                # First we check for unambiguous symbols (KiB, MiB, GiB, etc)
+                # and names (kibibyte, mebibyte, gibibyte, etc) because their
+                # handling is always the same.
+                if normalized_unit in (unit.binary.symbol.lower(), unit.binary.name.lower()):
+                    return int(tokens[0] * unit.binary.divider)
+                # Now we will deal with ambiguous prefixes (K, M, G, etc),
+                # symbols (KB, MB, GB, etc) and names (kilobyte, megabyte,
+                # gigabyte, etc) according to the caller's preference.
+                if (normalized_unit in (unit.decimal.symbol.lower(), unit.decimal.name.lower()) or
+                        normalized_unit.startswith(unit.decimal.symbol[0].lower())):
+                    return int(tokens[0] * (unit.binary.divider if binary else unit.decimal.divider))
+    # We failed to parse the size specification.
+    return None
+
+
 def input_type_check(data: object) -> None:
     """Ensure input data is a string. Raises `TypeError` if not."""
     if not isinstance(data, str):
         raise TypeError("Input data must be a 'str' object.")
 
 
+def _lazy_splitlines(text: str) -> Iterable[str]:
+    NEWLINES_PATTERN: str = r'(\r\n|\r|\n)'
+    NEWLINES_RE = re.compile(NEWLINES_PATTERN)
+    start = 0
+    for m in NEWLINES_RE.finditer(text):
+        begin, end = m.span()
+        if begin != start:
+            yield text[start:begin]
+        else:
+            yield ''
+        start = end
+
+    if text[start:]:
+        yield text[start:]
+
+
+def line_slice(
+        data: Union[str, Iterable[str], TextIO, bytes, None],
+        slice_start: Optional[int] = None,
+        slice_end: Optional[int] = None
+) -> Union[str, Iterable[str], TextIO, bytes, None]:
+    """
+    Slice input data by lines - lazily, if possible.
+
+    Accepts a string (for normal parsers) or an iterable (for streaming
+    parsers). Uses normal start/stop slicing values, but will always slice
+    on lines instead of characters. Positive slices will use less memory as
+    the function will attempt to lazily iterate over the input. A negative
+    slice parameter will force the function to read in all of the data and
+    then slice, which will use more memory.
+
+    Parameters:
+
+        data:              (string or iterable) - input to slice by lines
+        slice_start:       (int) - starting line
+        slice_end:         (int) - ending line
+
+    Returns:
+        string if input is a string.
+        iterable of strings if input is an iterable (for streaming parsers)
+    """
+    if not slice_start is None or not slice_end is None:
+        # standard parsers UTF-8 input
+        if isinstance(data, str):
+            data_iter = _lazy_splitlines(data)
+
+            # positive slices
+            if (slice_start is None or slice_start >= 0) \
+                and (slice_end is None or slice_end >= 0):
+
+                return '\n'.join(islice(data_iter, slice_start, slice_end))
+
+            # negative slices found (non-lazy, uses more memory)
+            else:
+                return '\n'.join(list(data_iter)[slice_start:slice_end])
+
+        # standard parsers bytes input
+        elif isinstance(data, bytes):
+            raise ValueError('Cannot slice bytes data.')
+
+        # streaming parsers UTF-8 input
+        else:
+            # positive slices
+            if (slice_start is None or slice_start >= 0) \
+                and (slice_end is None or slice_end >= 0) \
+                and data:
+
+                return islice(data, slice_start, slice_end)
+
+            # negative slices found (non-lazy, uses more memory)
+            elif data:
+                return list(data)[slice_start:slice_end]
+
+    return data
+
+
 class timestamp:
     __slots__ = ('string', 'format', 'naive', 'utc', 'iso')
 
@@ -408,9 +661,11 @@ class timestamp:
             {'id': 1710, 'format': '%m/%d/%Y, %I:%M:%S %p UTC%z', 'locale': None},  # Windows english format with UTC tz (found in systeminfo cli output): 3/22/2021, 1:15:51 PM (UTC+0000)
             {'id': 1750, 'format': '%Y/%m/%d-%H:%M:%S.%f', 'locale': None},  # Google Big Table format with no timezone: 1970/01/01-01:00:00.000000
             {'id': 1755, 'format': '%Y/%m/%d-%H:%M:%S.%f%z', 'locale': None},  # Google Big Table format with timezone: 1970/01/01-01:00:00.000000+00:00
+            {'id': 1760, 'format': '%Y-%m-%d %H:%M:%S%z', 'locale': None},  # certbot format with timezone: 2023-06-12 01:35:30+00:00
             {'id': 1800, 'format': '%d/%b/%Y:%H:%M:%S %z', 'locale': None},  # Common Log Format: 10/Oct/2000:13:55:36 -0700
             {'id': 2000, 'format': '%a %d %b %Y %I:%M:%S %p %Z', 'locale': None},  # en_US.UTF-8 local format (found in upower cli output): Tue 23 Mar 2021 04:12:11 PM UTC
             {'id': 3000, 'format': '%a %d %b %Y %I:%M:%S %p', 'locale': None},  # en_US.UTF-8 local format with non-UTC tz (found in upower cli output): Tue 23 Mar 2021 04:12:11 PM IST
+            {'id': 3500, 'format': '%a, %d %b %Y %H:%M:%S %Z', 'locale': None},  # HTTP header time format (always GMT so assume UTC): Wed, 31 Jan 2024 00:39:28 GMT
             {'id': 4000, 'format': '%A %d %B %Y %I:%M:%S %p %Z', 'locale': None},  # European-style local format (found in upower cli output): Tuesday 01 October 2019 12:50:41 PM UTC
             {'id': 5000, 'format': '%A %d %B %Y %I:%M:%S %p', 'locale': None},  # European-style local format with non-UTC tz (found in upower cli output): Tuesday 01 October 2019 12:50:41 PM IST
             {'id': 6000, 'format': '%a %b %d %I:%M:%S %p %Z %Y', 'locale': None},  # en_US.UTF-8 format (found in date cli): Wed Mar 24 06:16:19 PM UTC 2021
@@ -429,7 +684,7 @@ class timestamp:
         )
 
         # from https://www.timeanddate.com/time/zones/
-        # only removed UTC timezone and added known non-UTC offsets
+        # only removed UTC & GMT timezones and added known non-UTC offsets
         tz_abbr: set[str] = {
             'A', 'ACDT', 'ACST', 'ACT', 'ACWST', 'ADT', 'AEDT', 'AEST', 'AET', 'AFT', 'AKDT',
             'AKST', 'ALMT', 'AMST', 'AMT', 'ANAST', 'ANAT', 'AQTT', 'ART', 'AST', 'AT', 'AWDT',
@@ -438,7 +693,7 @@ class timestamp:
             'CHOT', 'CHUT', 'CIDST', 'CIST', 'CKT', 'CLST', 'CLT', 'COT', 'CST', 'CT', 'CVT', 'CXT',
             'ChST', 'D', 'DAVT', 'DDUT', 'E', 'EASST', 'EAST', 'EAT', 'ECT', 'EDT', 'EEST', 'EET',
             'EGST', 'EGT', 'EST', 'ET', 'F', 'FET', 'FJST', 'FJT', 'FKST', 'FKT', 'FNT', 'G',
-            'GALT', 'GAMT', 'GET', 'GFT', 'GILT', 'GMT', 'GST', 'GYT', 'H', 'HDT', 'HKT', 'HOVST',
+            'GALT', 'GAMT', 'GET', 'GFT', 'GILT', 'GST', 'GYT', 'H', 'HDT', 'HKT', 'HOVST',
             'HOVT', 'HST', 'I', 'ICT', 'IDT', 'IOT', 'IRDT', 'IRKST', 'IRKT', 'IRST', 'IST', 'JST',
             'K', 'KGT', 'KOST', 'KRAST', 'KRAT', 'KST', 'KUYT', 'L', 'LHDT', 'LHST', 'LINT', 'M',
             'MAGST', 'MAGT', 'MART', 'MAWT', 'MDT', 'MHT', 'MMT', 'MSD', 'MSK', 'MST', 'MT', 'MUT',
@@ -494,6 +749,9 @@ class timestamp:
         # UTC can also be indicated with 'Z' for Zulu time (ISO-8601). Convert to 'UTC'
         data = data.replace('Z', 'UTC')
 
+        # GMT and UTC are practically equivalent. Convert to 'UTC'
+        data = data.replace('GMT', 'UTC')
+
         if 'UTC' in data:
             utc_tz = True
             if 'UTC+' in data or 'UTC-' in data:
@@ -504,6 +762,7 @@ class timestamp:
              or '+00:00' in data \
              or '-00:00' in data:
             utc_tz = True
+            data = data.replace('+00:00', '+0000')  # fix for python 3.6
 
         # normalize the timezone by taking out any timezone reference, except UTC
         cleandata = data.replace('(', '').replace(')', '')
